from __future__ import annotations

from collections.abc import Sequence
from datetime import datetime

from pydantic import Field

from no_llm.config import (
    ConfigurableModelParameters,
    IntegrationAliases,
    ModelCapability,
    ModelConstraints,
    ModelIdentity,
    ModelMetadata,
    ModelMode,
    ModelPricing,
    ModelProperties,
    ParameterValue,
    ParameterVariant,
    PrivacyLevel,
    QualityProperties,
    RangeValidation,
    SpeedProperties,
    TokenPrices,
)
from no_llm.config.parameters import NotGiven
from no_llm.models.groq.base import GroqBaseConfiguration
from no_llm.providers import GroqProvider, Providers


class GroqMixtralConfiguration(GroqBaseConfiguration):
    """Configuration for Groq Mixtral model"""

    identity: ModelIdentity = ModelIdentity(
        id="groq-mixtral",
        name="Groq Mixtral",
        version="2024.02",
        description="Ultra-fast version of Mixtral optimized for rapid responses while maintaining high quality.",
        creator="Mistral",
    )

    providers: Sequence[Providers] = [GroqProvider()]

    mode: ModelMode = ModelMode.CHAT

    capabilities: set[ModelCapability] = {
        ModelCapability.STREAMING,
        ModelCapability.FUNCTION_CALLING,
        ModelCapability.JSON_MODE,
        ModelCapability.SYSTEM_PROMPT,
        ModelCapability.PARALLEL_FUNCTION_CALLING,
    }

    constraints: ModelConstraints = ModelConstraints(max_input_tokens=32768, max_output_tokens=32768)

    properties: ModelProperties | None = ModelProperties(
        speed=SpeedProperties(score=248.6, label="High", description="Average (1-3 seconds)"),
        quality=QualityProperties(score=76.0, label="High", description="High Quality"),
    )

    metadata: ModelMetadata = ModelMetadata(
        privacy_level=[PrivacyLevel.BASIC],
        pricing=ModelPricing(token_prices=TokenPrices(input_price_per_1k=0.00024, output_price_per_1k=0.00024)),
        release_date=datetime(2024, 3, 1),
        data_cutoff_date=datetime(2024, 1, 1),
    )

    integration_aliases: IntegrationAliases | None = IntegrationAliases(
        pydantic_ai="groq-mixtral",
        litellm="groq/mixtral-8x7b-32768",
        langfuse="groq-mixtral",
        lmarena="mixtral-8x7b-instruct-v0.1",
    )

    class Parameters(ConfigurableModelParameters):
        model_config = ConfigurableModelParameters.model_config
        temperature: ParameterValue[float | NotGiven] = Field(
            default_factory=lambda: ParameterValue[float | NotGiven](
                variant=ParameterVariant.VARIABLE,
                value=0.0,
                validation_rule=RangeValidation(min_value=0.0, max_value=1.0),
            )
        )

    parameters: Parameters = Field(default_factory=Parameters)  # type: ignore



from __future__ import annotations

from typing import TYPE_CHECKING

from pydantic_ai.models.openai import OpenAIModelSettings

from no_llm import ModelCapability
from no_llm.config import (
    ModelConfiguration,
)
from no_llm.config.parameters import NOT_GIVEN

if TYPE_CHECKING:
    from pydantic_ai.models import Model
    from pydantic_ai.settings import ModelSettings


class OpenaiBaseConfiguration(ModelConfiguration):
    def to_pydantic_model(self) -> Model:
        return super().to_pydantic_model()

    def to_pydantic_settings(self) -> ModelSettings:
        base = super().to_pydantic_settings()
        reasoning_effort = base.pop("reasoning_effort", "off")
        if ModelCapability.REASONING in self.capabilities and reasoning_effort not in [
            None,
            "off",
            NOT_GIVEN,
        ]:
            return OpenAIModelSettings(
                **base,
                openai_reasoning_effort=reasoning_effort,  # type: ignore
            )  # type: ignore
        return base



from __future__ import annotations

from typing import TYPE_CHECKING

from no_llm.config import (
    ModelConfiguration,
)

if TYPE_CHECKING:
    from pydantic_ai.models import Model
    from pydantic_ai.settings import ModelSettings


class LlamaBaseConfiguration(ModelConfiguration):
    def to_pydantic_model(self) -> Model:
        return super().to_pydantic_model()

    def to_pydantic_settings(self) -> ModelSettings:
        return super().to_pydantic_settings()



from __future__ import annotations

from typing import TYPE_CHECKING

from pydantic_ai.models.gemini import GeminiModelSettings, ThinkingConfig

from no_llm import ModelCapability
from no_llm.config import (
    ModelConfiguration,
)
from no_llm.config.parameters import NOT_GIVEN

if TYPE_CHECKING:
    from pydantic_ai.models import Model
    from pydantic_ai.settings import ModelSettings

THINKING_BUDGET = {
    "low": 512,
    "medium": 1024,
    "high": 4096,
}


class GeminiBaseConfiguration(ModelConfiguration):
    def to_pydantic_model(self) -> Model:
        return super().to_pydantic_model()

    def to_pydantic_settings(self) -> ModelSettings:
        base = super().to_pydantic_settings()
        reasoning_effort = base.pop("reasoning_effort", "off")
        if ModelCapability.REASONING not in self.capabilities:
            return base
        elif reasoning_effort in ["off", NOT_GIVEN]:
            include_thoughts = False
            thinking_budget = 0
        else:
            include_thoughts = True
            thinking_budget = THINKING_BUDGET[reasoning_effort]  # type: ignore
        return GeminiModelSettings(
            **base,
            gemini_thinking_config=ThinkingConfig(include_thoughts=include_thoughts, thinking_budget=thinking_budget),
        )
